import _extends from "@babel/runtime/helpers/esm/extends";
import _objectWithoutPropertiesLoose from "@babel/runtime/helpers/esm/objectWithoutPropertiesLoose";
import { jsx as _jsx } from "react/jsx-runtime";
import { Children, createRef, cloneElement, useRef, useState } from 'react';
import cx from 'classnames';
import { moleculeDropdownListSizes as SIZES } from '@s-ui/react-molecule-dropdown-list';
import { inputTypes } from '@s-ui/react-atom-input';
import MoleculeAutosuggestSingleSelection from './components/SingleSelection';
import MoleculeAutosuggestMultipleSelection from './components/MultipleSelection';
import { withOpenToggle } from '@s-ui/hoc';
import { getTarget } from '@s-ui/js/lib/react';
import { getCurrentElementFocused } from '@s-ui/js/lib/dom';
var BASE_CLASS = "sui-MoleculeAutosuggest";
var CLASS_FOCUS = BASE_CLASS + "--focus";
var CLASS_DISABLED = BASE_CLASS + "--disabled";
var AUTOSUGGEST_STATES = {
  ERROR: 'error',
  SUCCESS: 'success',
  ALERT: 'alert'
};

var getIsTypeableKey = function getIsTypeableKey(key) {
  var keysEdit = ['Backspace', 'Enter', 'Meta', 'Shift', 'ArrowLeft', 'ArrowRight'];
  return key.length === 1 || keysEdit.includes(key);
};

var MoleculeAutosuggest = function MoleculeAutosuggest(_ref) {
  var _cx;

  var children = _ref.children,
      disabled = _ref.disabled,
      errorState = _ref.errorState,
      _ref$id = _ref.id,
      id = _ref$id === void 0 ? '' : _ref$id,
      isOpen = _ref.isOpen,
      keysCloseList = _ref.keysCloseList,
      keysSelection = _ref.keysSelection,
      multiselection = _ref.multiselection,
      onBlur = _ref.onBlur,
      onChange = _ref.onChange,
      onEnter = _ref.onEnter,
      onFocus = _ref.onFocus,
      onToggle = _ref.onToggle,
      refMoleculeAutosuggestFromProps = _ref.refMoleculeAutosuggest,
      state = _ref.state,
      restProps = _objectWithoutPropertiesLoose(_ref, ["children", "disabled", "errorState", "id", "isOpen", "keysCloseList", "keysSelection", "multiselection", "onBlur", "onChange", "onEnter", "onFocus", "onToggle", "refMoleculeAutosuggest", "state"]);

  var refMoleculeAutosuggest = useRef(refMoleculeAutosuggestFromProps == null ? void 0 : refMoleculeAutosuggestFromProps.current);
  var refsMoleculeAutosuggestOptions = useRef([]);
  var refMoleculeAutosuggestInput = useRef();

  var _useState = useState(false),
      focus = _useState[0],
      setFocus = _useState[1];

  var extendedChildren = Children.toArray(children).filter(Boolean).map(function (child, index) {
    refsMoleculeAutosuggestOptions.current[index] = /*#__PURE__*/createRef();
    return /*#__PURE__*/cloneElement(child, {
      innerRef: refsMoleculeAutosuggestOptions.current[index],
      onSelectKey: keysSelection
    });
  });
  var className = cx(BASE_CLASS, errorState && BASE_CLASS + "--" + AUTOSUGGEST_STATES.ERROR, errorState === false && BASE_CLASS + "--" + AUTOSUGGEST_STATES.SUCCESS, state && BASE_CLASS + "--" + state, (_cx = {}, _cx[CLASS_FOCUS] = focus, _cx[CLASS_DISABLED] = disabled, _cx));

  var closeList = function closeList(ev) {
    var domMoleculeAutosuggest = refMoleculeAutosuggest.current;
    onToggle(ev, {
      isOpen: false
    });
    if (multiselection) onChange(ev, {
      value: ''
    });
    domMoleculeAutosuggest && !focus && domMoleculeAutosuggest.focus();
    setFocus(false);
    ev.preventDefault();
    ev.stopPropagation();
  };

  var focusFirstOption = function focusFirstOption(ev, _ref2) {
    var options = _ref2.options;
    if (options[0]) options[0].focus();
    ev.preventDefault();
    ev.stopPropagation();
  };

  var handleKeyDown = function handleKeyDown(ev) {
    ev.persist();
    var domInnerInput = refMoleculeAutosuggestInput.current;
    var domMoleculeAutosuggest = refMoleculeAutosuggest.current;
    var optionsFromRef = refsMoleculeAutosuggestOptions.current;
    var key = ev.key;
    var options = optionsFromRef.map(getTarget);
    var isTypeableKey = getIsTypeableKey(key);
    var isSelectionKey = keysSelection.includes(key);

    if (isTypeableKey) {
      if (!isSelectionKey) domInnerInput.focus();
    } else domMoleculeAutosuggest.focus();

    if (isOpen) {
      var currentElementFocused = getCurrentElementFocused();
      var isSomeOptionFocused = [].concat(options).includes(currentElementFocused);
      if (keysCloseList.includes(key)) closeList(ev);else if (key === 'ArrowDown' && !isSomeOptionFocused) focusFirstOption(ev, {
        options: options
      });else if (isSomeOptionFocused) handleFocusIn(ev);
    } else {
      if (key === 'Enter') {
        onEnter(ev);
      }
    }
  };

  var handleFocusIn = function handleFocusIn(ev) {
    onFocus(ev);
    setFocus(true);
  };

  var handleFocusOut = function handleFocusOut(ev) {
    ev.persist();
    var domContainer = refMoleculeAutosuggest.current;
    var domInnerInput = refMoleculeAutosuggestInput.current;
    var optionsFromRef = refsMoleculeAutosuggestOptions.current;
    var options = optionsFromRef.map(getTarget);
    setTimeout(function () {
      var currentElementFocused = getCurrentElementFocused();
      var focusOutFromOutside = ![domInnerInput].concat(options).includes(currentElementFocused) && !domContainer.contains(currentElementFocused);

      if (focusOutFromOutside) {
        if (isOpen) {
          closeList(ev);
        } else {
          setFocus(false);
          onBlur();
        }
      }
    }, 1);
    setFocus(true);
  };

  var handleInputKeyDown = function handleInputKeyDown(ev) {
    var key = ev.key;
    if (key !== 'ArrowDown') ev.stopPropagation();

    if (key === 'Enter') {
      onEnter(ev);
      closeList(ev);
    }
  };

  var handleClick = function handleClick() {
    (refMoleculeAutosuggestInput == null ? void 0 : refMoleculeAutosuggestInput.current) && refMoleculeAutosuggestInput.current.focus();
  };

  var autosuggestSelectionProps = _extends({
    refMoleculeAutosuggestFromProps: refMoleculeAutosuggestFromProps,
    onToggle: onToggle,
    onChange: onChange,
    onBlur: onBlur,
    onEnter: onEnter,
    onFocus: onFocus,
    isOpen: isOpen,
    keysCloseList: keysCloseList,
    keysSelection: keysSelection,
    disabled: disabled,
    errorState: errorState,
    state: state,
    id: id,
    onInputKeyDown: handleInputKeyDown,
    refMoleculeAutosuggest: refMoleculeAutosuggest,
    innerRefInput: refMoleculeAutosuggestInput,
    children: extendedChildren
  }, restProps);

  var AutosuggestSelection = multiselection ? MoleculeAutosuggestMultipleSelection : MoleculeAutosuggestSingleSelection;
  return /*#__PURE__*/_jsx("div", {
    ref: refMoleculeAutosuggest,
    tabIndex: "0",
    className: className,
    onKeyDown: handleKeyDown,
    onFocus: handleFocusIn,
    onBlur: handleFocusOut,
    onClick: handleClick,
    role: "combobox",
    "aria-controls": id,
    "aria-expanded": isOpen,
    children: /*#__PURE__*/_jsx(AutosuggestSelection, _extends({}, autosuggestSelectionProps))
  });
};

MoleculeAutosuggest.defaultProps = {
  onChange: function onChange() {},
  onBlur: function onBlur() {},
  onToggle: function onToggle() {},
  onEnter: function onEnter() {},
  onSelect: function onSelect() {},
  onFocus: function onFocus() {},
  keysSelection: [' ', 'Enter'],
  keysCloseList: ['Escape']
};
var MoleculeAutoSuggestWithOpenToggle = withOpenToggle(MoleculeAutosuggest);
MoleculeAutoSuggestWithOpenToggle.displayName = 'MoleculeAutosuggest';
export default MoleculeAutoSuggestWithOpenToggle;
export { SIZES as MoleculeAutosuggestDropdownListSizes };
export { AUTOSUGGEST_STATES as MoleculeAutosuggestStates };